# The MIT License (MIT)
# Supplementary Material to Article : 
# P. A. Araya Melo, M. Crucifix, N. Bounceur, 
# Global sensitivity analysis of Indian Monsoon during the Pleistocene
# http://www.clim-past-discuss.net/10/1609/2014/cpd-10-1609-2014.html
# 
# Copyright (c) 2014 Michel Crucifix
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 

pdf('Empirical_Distribution.pdf', width=6, height=6, pointsize=15)

verbose=TRUE 
require(pleistocene) 

  rate = (44.0095/28.965338)*10^(-6)
  minco2 = 180*rate
  maxco2 = 280*rate

  ice1 = 1
  ice2 = 11

  pi180 = pi / 180.
  shifts <- c(ecosw=-0.05, esinw=-0.05, eps=22 * pi180, co2=minco2, ice=ice1)
  scales <- c(ecosw=0.10, esinw=0.10, eps=3 * pi180, co2=(maxco2 - minco2) , ice=(ice2-ice1) )

 n = 60
 levels <- list(co2=seq(0,1,l=n), ice=seq(0,1,l=n))
 sqrt3 <- 1./sqrt(3.)
 rhoiceco2 <- outer(levels$co2, levels$ice,  
                    function(co2,ice) 
                    { 
                      # rescaling as Pablo uses [0,1] intervals, rather
                      # than [-1,1] in the orginal development
                      c <- 2 * co2 - 1
                      i <- 2 * ice - 1
                      dnorm (sqrt3*(2*c + i)  )   *
                      dnorm (sqrt3*(+c + 2*i) ) 
                    }
                   )

 rhoiceco2 = rhoiceco2 / sum(rhoiceco2)
 
 OUT = rhoiceco2[order(rhoiceco2, decreasing = TRUE)]

 cont = c(1.462948E-4, 1.894344E-4, 3.118016E-4)


 data(edcCO2)
 data(LR04)

 ind = which(LR04$Age.ka < 800)
 Ice = LR04[ind, c('Age.ka', 'd18O') ]
 CO2 = approx(edcCO2$CE, edcCO2$CO2, - Ice$Age.ka * 1000)$y

 


 data(edcCO2)
 data(LR04)


x = ( levels$co2 * scales['co2'] + shifts['co2'] ) / rate
y = ( levels$ice * scales['ice'] + shifts['ice'] )

Ice$Ice = ( Ice$d18O -3.1 ) /2 * 10

contour(x, y, z=rhoiceco2, levels=cont, labels=c('95%','90%','66%'), lwd=2, xlab='CO2 [ppm]', ylab='Ice Level')
 lines(CO2, Ice$Ice, type='p')

dev.off()

