# The MIT License (MIT)
# Supplementary Material to Article : 
# P. A. Araya Melo, M. Crucifix, N. Bounceur, 
# Global sensitivity analysis of Indian Monsoon during the Pleistocene
# http://www.clim-past-discuss.net/10/1609/2014/cpd-10-1609-2014.html
# 
# Copyright (c) 2014 Michel Crucifix
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 

require(GP)

# load experiment data


optimise_lambda <- function(DATA)
{
  
  X = DATA$X
  Y = DATA$Y
  
  OPTIMISED = list()
  lambdas = list()
    
  # start (guess) values 
  lambdas$temp2$theta=c(0.73,0.90,1.0,7.8,0.35)
  lambdas$temp2$nugget=0.00350
  
  lambdas$precip2$theta=c(1.2,2.7,6.5,100,0.23)
  lambdas$precip2$nugget=0.019
  
  lambdas$sst$theta=c(0.35,0.16,6.0,100,0.26)
  lambdas$sst$nugget=0.01
  
  lambdas$logml$theta=c(0.17,0.11,1.50,100, 0.21)
  lambdas$logml$nugget=0.01
  
  
  SL = function(optimvec) {
  evec = exp(optimvec)
  lambdas[[var]]$theta = pmin(evec[1:5], 100)
  lambdas[[var]]$nugget = max(evec[6], 1.e-4)
  E = -GP_C(X, Y[,var], lambdas[[var]], covar=exp)$log_pen_REML
  print(E)
  E
  }
  
   for (  var in c('temp2','precip2','sst','logml') )
    {
      evec0 = log(c(lambdas[[var]]$theta, lambdas[[var]]$nugget))
      out = exp(optim(evec0,SL )$par)
      OPTIMISED[[var]] <- list (theta = out[1:5], nugget = out[6])
      names(OPTIMISED[[var]]$theta) <- colnames(X)
    }
  return(OPTIMISED)
}


# -----------------------------------------

RDatapath = file.path('..','Data','RData')
load(file.path(RDatapath, 'exp_output.rda') )
Lambdas = lapply(DATA, optimise_lambda)
save('Lambdas', file = file.path(RDatapath, lambda.rda') )

# -----------------------------------------


# OPTIMISED
#$precip2
#$precip2$theta
#      esinw       ecosw         obl         co2         ice
#  1.1534407   3.0370232   6.7403597 125.8860400   0.2280708 --> OK
#
#$precip2$nugget
#[1] 0.01883809
#
#
#$temp2
#$temp2$theta
#    esinw     ecosw       obl       co2       ice
#0.7035516 0.9144678 1.0635433 9.4052954 0.3347619 ---> OK
#
#$temp2$nugget
#[1] 0.004692434
#
#
#$sst
#$sst$theta
#      esinw       ecosw         obl         co2         ice
#  1.1177969   1.1417100 200.0787951  97.8597018   0.7306662 --> OK
#
#$sst$nugget
#[1] 0.003485255
#
#
#$logml
#$logml$theta
#    esinw     ecosw       obl       co2       ice
#0.7665467 0.3081668 1.2574839 4.1140545 1.0696023
#
#$logml$nugget
#[1] 0.0439074 --> OK
