# The MIT License (MIT)
# Supplementary Material to Article : 
# P. A. Araya Melo, M. Crucifix, N. Bounceur, 
# Global sensitivity analysis of Indian Monsoon during the Pleistocene
# http://www.clim-past-discuss.net/10/1609/2014/cpd-10-1609-2014.html
# 
# Copyright (c) 2014 Michel Crucifix
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 

# R CODE FOR READING AND PACKAGING DATA GENERATED BY HADCM3


# scale design to be compatible with Pablo's supplied lambdas
# see his email 07 octobre 2013
scale_design <- function(X)
{
  rate = (44.0095/28.965338)*10^(-6)
  minco2 = 180*rate
  maxco2 = 280*rate

  ice1 = 1
  ice2 = 11

  pi180 = pi / 180.
  shifts <- c(ecosw=-0.05, esinw=-0.05, eps=22 * pi180, co2=minco2, ice=ice1)
  scales <- c(ecosw=0.10, esinw=0.10, eps=3 * pi180, co2=(maxco2 - minco2) , ice=(ice2-ice1) )
  for ( i in colnames (X)) { X[,i] = (X[,i] - shifts[i] ) / scales[i] }
  attr(X,'scales') <- scales
  attr(X,'shifts') <- shifts
  X
}


# the following is deprecated
scale_design_old <- function(X)
{
  shifts <- c(esinw=0.00, ecosw=0.00, eps=0.40701, co2=0.00035, ice=5.5)
  scales <- c(esinw=0.04, ecosw=0.04, eps=0.02138, co2=8.0E-5, ice=5)
  for ( i in colnames (X)) { X[,i] = (X[,i] - shifts[i] ) / scales[i] }
  attr(X,'scales') <- scales
  attr(X,'shifts') <- shifts
  X
}


read_data <- function(filename,nrow)
{
  zz <- file(filename, "rb")
  cat(sprintf('Reading  file %s \n Column',  filename))
  Data <-  array(0,dim=c(nrow,9))
  for (i in 1:nrow)
   {
     cat(sprintf(' %i,  ', i))
     Data[i,] <- readBin(zz,numeric(),n=9,size=4)
   }
  cat('ok. Closing connection \n ')
  close(zz)
  X_orig <- Data[,1:5];  colnames(X_orig) <- c('ecosw','esinw','eps','co2','ice')
  Y <- Data[,6:9];  colnames(Y) <- c('temp2','precip2','sst','logml')
  X <- scale_design(X_orig)
  Y[,'logml'] = log(Y[,'logml'])
  return(list(X=X, X_orig=X_orig, Y=Y))
}


 DATA = list()
 local({
 RAWDATADIR = file.path('..', 'Data', 'Raw' )
 print(RAWDATADIR)
 OUT <- read_data(file.path(RAWDATADIR, 'Orog_data' , 'data.bin'),61 ) ; 
 DATA[["Orog_full"]] <<- OUT 

 OUT$X_orig = OUT$X_orig[-c(11,40),]
 attr_ = attributes(OUT$X)
 OUT$X = OUT$X[-c(11,40),]
 mostattributes(OUT$X) <- attr_
 dim(OUT$X) = c(59,5)
 OUT$Y = OUT$Y[-c(11,40),]

 DATA[["Orog_without_11_40"]] <<- OUT 
 OUT <- read_data(file.path(RAWDATADIR, 'No_Orog_data' , 'data.bin'),57 ) ; 


 DATA[["No_Orog"]] <<-  OUT 

 })

# helper function to print data

save_data <- function(ITEM) 
{
 table = cbind(DATA[[ITEM]]$X, DATA[[ITEM]]$Y)
 write.table(file=sprintf(file.path('..','Data','Txt','%s.txt'), ITEM), table, row.names="FALSE")
}

RDatapath = file.path('..','Data','RData')
save(DATA, file=file.path(RDatapath, 'exp_output.rda') 

